package com.jxl.mobile.charts
{
	import com.jxl.mobile.charts.vo.ChartVO;
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.BlendMode;
	import flash.display.Graphics;
	import flash.display.GraphicsBitmapFill;
	import flash.display.GraphicsEndFill;
	import flash.display.GraphicsPath;
	import flash.display.GraphicsPathCommand;
	import flash.display.GraphicsSolidFill;
	import flash.display.GraphicsStroke;
	import flash.display.IGraphicsData;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.utils.getTimer;
	
	import mx.collections.ArrayCollection;
	import mx.core.UIComponent;
	
	public class PlotChartGraphicsDataVector extends UIComponent
	{
		
		[ArrayElementType("com.jxl.mobile.charts.vo.ChartVO")]
		private var _dataProvider:ArrayCollection;
		private var dataDirty:Boolean = false;
		
		private var _xMinimum:Number = 0;
		private var _xMaximum:Number = 100;
		private var _yMinimum:Number = 0;
		private var _yMaximum:Number = 100;
		private var axisDirty:Boolean = false;
		
		private var startTime:Number;
		private var elapsedTime:Number;
		private var currentDrawIndex:int;
		private var drawing:Boolean = false;
		private const MAX_TIME:Number = 10;
		private var timer:Sprite;
		private var frameCallback:Function;
		
		private var shape:Shape;
		private var drawingData:Vector.<IGraphicsData> = new Vector.<IGraphicsData>();
		private var pooledX:Number = 0;
		private var pooledY:Number = 0;
		private var pooledFill:GraphicsSolidFill =  new GraphicsSolidFill(0xFF0000);
		private var pooledPath:GraphicsPath = new GraphicsPath(new Vector.<int>(), new Vector.<Number>());
		
		
		[Bindable]
		public function get dataProvider():ArrayCollection { return _dataProvider; }
		public function set dataProvider(value:ArrayCollection):void
		{
			_dataProvider = value;
			dataDirty = true;
			invalidateProperties();
		}
		
		public function get xMinimum():Number { return _xMinimum; }
		public function set xMinimum(value:Number):void	
		{
			_xMinimum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function get xMaximum():Number { return _xMaximum; }
		public function set xMaximum(value:Number):void	
		{
			_xMaximum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function get yMinimum():Number { return _yMinimum; }
		public function set yMinimum(value:Number):void	
		{
			_yMinimum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function get yMaximum():Number { return _yMaximum; }
		public function set yMaximum(value:Number):void	
		{
			_yMaximum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function PlotChartGraphicsDataVector()
		{
			super();
			
			width = 300;
			height = 300;
			
			pooledPath.commands.push(GraphicsPathCommand.MOVE_TO, GraphicsPathCommand.LINE_TO, GraphicsPathCommand.LINE_TO, GraphicsPathCommand.LINE_TO);
			drawingData.push(pooledFill, pooledPath);
		}
		
		protected override function createChildren():void
		{
			super.createChildren();
			shape = new Shape();
			addChild(shape);
		}
		
		protected override function commitProperties():void
		{
			super.commitProperties();
			
			if(dataDirty || axisDirty)
			{
				dataDirty = false;
				redraw();
			}
		}
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			var g:Graphics = graphics;
			g.clear();
			g.lineStyle(0, 0xFF0000);
			g.drawRect(0, 0, width, height);
			g.endFill();
		}
		
		private function stopDrawing():void
		{
			drawing = false;
			if(timer)
				timer.removeEventListener(Event.ENTER_FRAME, onTick);
		}
		
		public function redraw():void
		{
			Profile.start("PlotChartGraphicsDataVector::redraw");
			
			if(drawing)
			{
				stopDrawing();
			}
			
			shape.graphics.clear();
			
			if(_dataProvider == null) return;
			
			// TODO: x & y min/max range check before attempting to redraw st00pid valuez
			
			const len:int = _dataProvider.length;
			if(len < 1) return;
			
			currentDrawIndex = 0;
			drawing = true;
			
			var chartVO:ChartVO = _dataProvider.getItemAt(10) as ChartVO;
			chartVO.xValue = getRandomNumber(0, 100);
			chartVO.yValue = getRandomNumber(0, 100);
			
			
			redrawNext();
		}
		
		private function getRandomNumber(min:Number, max:Number):Number
		{
			var value:Number = (Math.random() * max) - min;
			if(value < min)
				value = min;
			return value;
		}
		
		private function redrawNext():void
		{
			//Debug.log("redrawNext, currentDrawIndex: " + currentDrawIndex);
			if(_dataProvider.length)
			{
				startTime = getTimer();
				const xRange:Number = xMaximum - xMinimum;
				const yRange:Number = yMaximum - yMinimum;
				const len:int = _dataProvider.length;
				for(var index:int = currentDrawIndex; index < len; index++)
				{
					var chartVO:ChartVO = _dataProvider.getItemAt(index) as ChartVO;
					
					pooledX = width * (chartVO.xValue / xRange);
					pooledY = height * (chartVO.yValue / yRange);
					
					pooledPath.data.length = 0;
					pooledPath.data.push(pooledX + 4,pooledY, pooledX + 4,pooledY + 4, pooledX,pooledY + 4, pooledX,pooledY);
					
					shape.graphics.drawGraphicsData(drawingData);
					
					elapsedTime = getTimer() - startTime;
					if(elapsedTime > MAX_TIME)
					{
						currentDrawIndex = index + 1;
						waitAFrame(redrawNext);
						return;
					}
				}
				Profile.stop("PlotChartGraphicsDataVector::redraw");
				
				
				stopDrawing();
			}
			else
			{
				Profile.stop("PlotChartGraphicsDataVector::redraw");
				
				stopDrawing();
			}
		}
		
		
		private function waitAFrame(callback:Function):void
		{
			frameCallback = callback;
			if(timer == null)
			{
				timer = new Sprite();
			}
			timer.addEventListener(Event.ENTER_FRAME, onTick);
		}
		
		private function onTick(event:Event):void
		{
			timer.removeEventListener(Event.ENTER_FRAME, onTick);
			if(drawing)
			{
				frameCallback();
			}
			
		}
		
	}
}