package com.jxl.mobile.charts
{
	import com.jxl.mobile.charts.vo.ChartVO;
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.BlendMode;
	import flash.display.Graphics;
	import flash.display.GraphicsBitmapFill;
	import flash.display.IGraphicsData;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.utils.getTimer;
	
	import mx.collections.ArrayCollection;
	import mx.core.UIComponent;
	
	import com.jxl.mobile.charts.vo.ChartVO;
	
	public class PlotChartVector extends UIComponent
	{
		
		[ArrayElementType("com.jxl.mobile.charts.vo.ChartVO")]
		private var _dataProvider:ArrayCollection;
		private var dataDirty:Boolean = false;
		
		private var _xMinimum:Number = 0;
		private var _xMaximum:Number = 100;
		private var _yMinimum:Number = 0;
		private var _yMaximum:Number = 100;
		private var axisDirty:Boolean = false;
		
		private var startTime:Number;
		private var elapsedTime:Number;
		private var currentDrawIndex:int;
		private var currentPointIndex:int;
		private var drawing:Boolean = false;
		private const MAX_TIME:Number = 10;
		private var timer:Sprite;
		private var frameCallback:Function;
		
		private var shapesHolder:Sprite;
		private var pointPool:Vector.<Point> = new Vector.<Point>();
		private var shapesPool:Vector.<Shape> = new Vector.<Shape>();
		private var pointsToDraw:Vector.<Point> = new Vector.<Point>();
		private var pooledX:Number = 0;
		private var pooledY:Number = 0;
		
		[Bindable]
		public function get dataProvider():ArrayCollection { return _dataProvider; }
		public function set dataProvider(value:ArrayCollection):void
		{
			_dataProvider = value;
			dataDirty = true;
			invalidateProperties();
		}
		
		public function get xMinimum():Number { return _xMinimum; }
		public function set xMinimum(value:Number):void	
		{
			_xMinimum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function get xMaximum():Number { return _xMaximum; }
		public function set xMaximum(value:Number):void	
		{
			_xMaximum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function get yMinimum():Number { return _yMinimum; }
		public function set yMinimum(value:Number):void	
		{
			_yMinimum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function get yMaximum():Number { return _yMaximum; }
		public function set yMaximum(value:Number):void	
		{
			_yMaximum = value;
			axisDirty = true;
			invalidateProperties();
		}
		
		public function PlotChartVector()
		{
			super();
			
			width = 300;
			height = 300;
		}
		
		protected override function createChildren():void
		{
			super.createChildren();
			shapesHolder = new Sprite();
			addChild(shapesHolder);
		}
		
		protected override function commitProperties():void
		{
			super.commitProperties();
			
			if(dataDirty || axisDirty)
			{
				dataDirty = false;
				redraw();
			}
		}
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			var g:Graphics = graphics;
			g.clear();
			g.lineStyle(0, 0xFF0000);
			g.drawRect(0, 0, width, height);
			g.endFill();
		}
		
		// RamerDouglasPeucker
		private function shortenPointList(pointList:Vector.<Point>, epsilon:Number):Vector.<Point>
		{
			var resultList:Vector.<Point>;
			var epsilon:Number = 0;
			//Find the point with the maximum distance
			var maximumDistance:Number = 0;
			const len:int = pointList.length;
			if(len == 0)
			{
				resultList = new Vector.<Point>();
				return resultList;
			}
			
			var distance:Number;
			var startIndex:int = 0;
			for(var index:int = 0; index < len; index += 2)
			{
				if(index + 1 < len)
				{
					var pt1:Point = pointList[index];
					var pt2:Point = pointList[index + 1];
					distance = Point.distance(pt1, pt2);
					if(distance > maximumDistance)
					{
						startIndex = index;
						maximumDistance = distance;
					}
				}
			}
			
			if(maximumDistance >= epsilon)
			{
				var recursiveResults1:Vector.<Point> = shortenPointList(pointList.slice(0, startIndex), epsilon);
				var recursiveResults2:Vector.<Point> = shortenPointList(pointList.slice(startIndex, pointList.length), epsilon);
				resultList = recursiveResults1.slice(0, len - 1).concat(recursiveResults2.slice(0, len));
			}
			else
			{
				resultList = new Vector.<Point>(pointList[0], pointList[len - 1]);
			}
			
			return resultList;
		}
		
		private function stopDrawing():void
		{
			drawing = false;
			if(timer)
				timer.removeEventListener(Event.ENTER_FRAME, onTick);
		}
		
		public function redraw():void
		{
			Profile.start("PlotChartVector::redraw");
			
			if(drawing)
			{
				stopDrawing();
			}
			
			removeAllShapes();
			removeAllPoints();
			
			if(_dataProvider == null) return;
			
			// TODO: x & y min/max range check before attempting to redraw st00pid valuez
			
			const len:int = _dataProvider.length;
			if(len < 1) return;
			
			currentDrawIndex = 0;
			currentPointIndex = 0;
			drawing = true;
			
			redrawNext();
		}
		
		private function removeAllShapes():void
		{
			var len:int = shapesHolder.numChildren;
			while(len--)
			{
				shapesPool.push(shapesHolder.removeChildAt(len) as Shape);
			}
		}
		
		private function removeAllPoints():void
		{
			var len:int = pointsToDraw.length;
			while(len--)
			{
				pointPool.push(pointsToDraw.pop());
			}
		}
		
		private function getShape():Shape
		{
			if(shapesPool.length > 0)
			{
				return shapesPool.pop();
			}
			else
			{
				return new Shape();
			}
		}
		
		private function redrawNext():void
		{
			//Debug.log("redrawNext, currentDrawIndex: " + currentDrawIndex);
			if(_dataProvider.length)
			{
				startTime = getTimer();
				const xRange:Number = xMaximum - xMinimum;
				const yRange:Number = yMaximum - yMinimum;
				const len:int = _dataProvider.length;
				for(var index:int = currentDrawIndex; index < len; index++)
				{
					var chartVO:ChartVO = _dataProvider.getItemAt(index) as ChartVO;
					
					pooledX = width * (chartVO.xValue / xRange);
					pooledY = height * (chartVO.yValue / yRange);
					var point:Point = getPoint(pooledX, pooledY);
					pointsToDraw[index] = point;
					
					elapsedTime = getTimer() - startTime;
					if(elapsedTime > MAX_TIME)
					{
						currentDrawIndex = index + 1;
						waitAFrame(redrawNext);
						return;
					}
				}
				
				redrawNextShape();
			}
			else
			{
				redrawNextShape();
			}
		}
		
		private function prepRedrawNextShape():void
		{
			/*
			Profile.start("shorten");
			Debug.log("Before: " + this.pointsToDraw.length);
			try
			{
				pointsToDraw = shortenPointList(pointsToDraw, 0);
			}
			catch(err:Error)
			{
				Debug.log("Failed to reduce point list: " + err);
			}
			Debug.log("After: " + this.pointsToDraw.length);
			Profile.stop("shorten");
			*/
			redrawNextShape();
		}
		
		private function redrawNextShape():void
		{
			//Debug.log("redrawNextShape, currentPointIndex: " + currentPointIndex);
			if(pointsToDraw.length > 0)
			{
				startTime = getTimer();
				const len:int = pointsToDraw.length;
				for(var index:int = currentPointIndex; index < len; index++)
				{
					var point:Point = pointsToDraw[index];
					var shape:Shape = getShape();
					addChild(shape);
					shape.x = point.x;
					shape.y = point.y;
					shape.graphics.beginFill(0xFF0000);
					//shape.graphics.drawCircle(0, 0, 3);
					shape.graphics.drawRect(0, 0, 4, 4);
					shape.graphics.endFill();
					
					elapsedTime = getTimer() - startTime;
					if(elapsedTime > MAX_TIME)
					{
						currentPointIndex = index + 1;
						waitAFrame(redrawNextShape);
						return;
					}
				}
				
				stopDrawing();
				Profile.stop("PlotChartVector::redraw");
			}
			else
			{
				stopDrawing();
				Profile.stop("PlotChartVector::redraw");
			}
		}
		
		private function getPoint(x:Number, y:Number):Point
		{
			var point:Point;
			if(pointPool.length > 0)
			{
				
				point = pointPool.pop();
			}
			else
			{
				point = new Point(x, y);
			}
			
			return point;
		}
		
		private function waitAFrame(callback:Function):void
		{
			frameCallback = callback;
			if(timer == null)
			{
				timer = new Sprite();
			}
			timer.addEventListener(Event.ENTER_FRAME, onTick);
		}
		
		private function onTick(event:Event):void
		{
			timer.removeEventListener(Event.ENTER_FRAME, onTick);
			if(drawing)
			{
				frameCallback();
			}
			
		}
		
	}
}